"use strict";
/**
 * This file was automatically generated by openapi-codegeneration.
 * DO NOT MODIFY IT BY HAND. Instead, modify the source OpenAPI file,
 * and run openapi-codegeneration to regenerate this file.
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.APIClient = exports.InvalidUrlError = exports.UnsuccessfulRequestError = exports.FetchError = exports.ValidationError = void 0;
/* eslint-disable @typescript-eslint/no-unused-vars */
/* eslint-disable @typescript-eslint/ban-ts-comment */
const types_js_1 = require("./types.js");
const requestValidation_js_1 = require("./requestValidation.js");
const cross_fetch_1 = __importDefault(require("cross-fetch"));
/**
 * This error class should be used if an error occurs during validation of a request/response.
 * @category Errors
 */
class ValidationError extends Error {
    constructor(message, errors) {
        super(message);
        this.name = "ValidationError";
        this.errors = errors;
    }
}
exports.ValidationError = ValidationError;
/**
 * This error class should be used if an error occurs while fetching a request.
 * @category Errors
 */
class FetchError extends Error {
    constructor(message) {
        super(message);
        this.name = "FetchError";
    }
}
exports.FetchError = FetchError;
/**
 * This error class should be used if the response of the server has a status
 * greater than or equal to 400. This error should contain the validated response.
 * @category Errors
 */
class UnsuccessfulRequestError extends Error {
    constructor(message, response) {
        super(message);
        this.response = response;
        this.name = 'UnsuccessfulRequestError';
    }
}
exports.UnsuccessfulRequestError = UnsuccessfulRequestError;
/**
 * This error class should be used if the validation of an url fails.
 * @category Errors
 */
class InvalidUrlError extends Error {
    constructor(message) {
        super(message);
        this.name = 'InvalidUrlError';
    }
}
exports.InvalidUrlError = InvalidUrlError;
/**
 * This function attempts to parse a Response as ResponseData.
 */
async function parseResponse(response) {
    let text = null;
    try {
        text = await response.text();
        const json = JSON.parse(text);
        return { status: response.status, body: json };
    }
    catch {
        return { status: response.status, body: text };
    }
}
/**
 * This function checks if a string is a valid http url.
 * @param string The string to be checked.
 * @returns True if the string is a valid http url.
 */
function isValidHttpUrl(string) {
    let url;
    try {
        url = new URL(string);
    }
    catch (_) {
        return false;
    }
    return url.protocol === 'http:' || url.protocol === 'https:';
}
/**
 * This function takes an url and a pattern for the requested endpoint and parses the contained path parameters.
 * @param url The url to be parsed.
 * @param endpoint Pattern of the requested endpoint.
 * @throws {InvalidUrlError} Thrown if the provided url does not end with the provided endpoint.
 * @returns An array containing all parsed path parameters in order of appearance.
 * @example
 * // returns ["username", "role_name"]
 * parsePathParameters("https://api.example.com/users/username/roles/role_name", "/users/{}/roles/{}")
 */
function parsePathParameters(url, endpoint) {
    const parameterRegex = '([a-zA-Z0-9-:]+)';
    const regex = new RegExp(endpoint.replaceAll('{}', parameterRegex) + '(?:.(?!\\\\))?$');
    const matches = url.match(regex);
    if (!matches)
        throw new InvalidUrlError('Url does not end with the provided endpoint');
    return matches.slice(1);
}
/**
 * This function validates a given http url and parses its path parameters.
 * @param url The url to be validated.
 * @param endpoint Pattern of the requested endpoint.
 * @throws {InvalidUrlError} Thrown if the validation of the provided url fails.
 * @returns An array containing all parsed path parameters in order of appearance (see {@link parsePathParameters}).
 * @example
 * // returns ["username", "role_name"]
 * validateUrl("https://api.example.com/users/username/roles/role_name", "/users/{}/roles/{}")
 */
function validateUrl(url, endpoint) {
    if (!isValidHttpUrl(url))
        throw new InvalidUrlError('Provided url is not a valid http url');
    return parsePathParameters(url, endpoint);
}
/**
 * This function appends a given endpoint to an url.
 * @param url The url to append the endpoint to.
 * @param endpoint The endpoint to be appended.
 * @example
 * // returns "https://api.example.com/devices/device_id/token"
 * appendToUrl("https://api.example.com/devices/device_id", "/token")
 * @example
 * // returns "https://api.example.com/devices/device_id/token"
 * appendToUrl("https://api.example.com/devices/device_id/", "/token")
 * @returns The url with the appended endpoint.
 */
function appendToUrl(url, endpoint) {
    return url.endsWith('/') ? `${url.slice(0, -1)}${endpoint}` : `${url}${endpoint}`;
}
/**
 * Client for the use of the Crosslab API
 * @category Client
 */
class APIClient {
    /**
     * @category Constructors
     */
    constructor(url, accessToken) {
        this.fetch = async (url, info) => {
            let raw_response;
            try {
                if (url.startsWith(this._url)) {
                    raw_response = await (0, cross_fetch_1.default)(url, info);
                }
                else {
                    raw_response = await (0, cross_fetch_1.default)(appendToUrl(this._url, "/proxy?" + new URLSearchParams([["URL", url]]).toString()), info);
                }
            }
            catch (error) {
                if (error instanceof Error) {
                    throw new FetchError(error.message);
                }
                else if (typeof error === "string") {
                    throw new FetchError(error);
                }
                else {
                    throw new FetchError("Something went wrong while trying to fetch the request");
                }
            }
            return await parseResponse(raw_response);
        };
        this._url = url.endsWith('/') ? url.slice(0, -1) : url;
        this._accessToken = accessToken ?? '';
    }
    /**
     * @category Accessors
     */
    get url() {
        return this._url;
    }
    /**
     * @category Accessors
     */
    set url(url) {
        this._url = url.endsWith('/') ? url.slice(0, -1) : url;
    }
    /**
     * @category Accessors
     */
    set accessToken(accessToken) {
        this._accessToken = accessToken;
    }
    /**
     * @category Accessors
     */
    get accessToken() {
        return this._accessToken;
    } /**
     * Login user
     *
     * @param username
     * Username of the user.
     * @param password
     * Password of the user.
     * @param options.method
     * Authentication method.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * Return the access token.
     *
     * @category Authentication Service Methods
     */
    async login(username, password, options) {
        const url = appendToUrl(this._url, "/login");
        const body = {
            username: username,
            password: password,
            method: options?.method,
        };
        if (!requestValidation_js_1.AuthenticationServiceRequestValidation.validateLoginInput(body))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.AuthenticationServiceRequestValidation.validateLoginInput.errors);
        const response = await this.fetch(url, {
            method: "POST",
            headers: [
                ["Content-Type", "application/json"],
                ...(options?.headers ?? [])
            ],
            body: JSON.stringify(body)
        });
        if (!requestValidation_js_1.AuthenticationServiceRequestValidation.validateLoginOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.AuthenticationServiceRequestValidation.validateLoginOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        this._accessToken = response.body;
    } /**
     * Logout user
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * Successfully logged out the user.
     *
     * @category Authentication Service Methods
     */
    async logout(options) {
        const url = appendToUrl(this._url, "/logout");
        const body = { token: this._accessToken };
        if (!requestValidation_js_1.AuthenticationServiceRequestValidation.validateLogoutInput(body))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.AuthenticationServiceRequestValidation.validateLogoutInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: "POST",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ],
            body: JSON.stringify(body)
        });
        if (!requestValidation_js_1.AuthenticationServiceRequestValidation.validateLogoutOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.AuthenticationServiceRequestValidation.validateLogoutOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        this._accessToken = '';
    } /**
     * Get all users
     *
     * @param options.username
     * filter for users with a specific username
     * @param options.url
     * Url of the Authentication Service to be used.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The JSON representation of the users.
     *
     * @category Authentication Service Methods
     */
    async listUsers(options) {
        const url = appendToUrl(options?.url ?? this._url, "/users");
        const parameters = {
            username: options?.username,
        };
        const query = [];
        if (parameters["username"])
            query.push(["username", parameters["username"].toString()]);
        if (!requestValidation_js_1.AuthenticationServiceRequestValidation.validateListUsersInput(parameters))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.AuthenticationServiceRequestValidation.validateListUsersInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url + '?' + new URLSearchParams(query), {
            method: "GET",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ]
        });
        if (!requestValidation_js_1.AuthenticationServiceRequestValidation.validateListUsersOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.AuthenticationServiceRequestValidation.validateListUsersOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    } /**
     * Create a user
     *
     * @param user
     * User to be created.
     * @param options.url
     * Url of the Authentication Service to be used.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The JSON representation of the created user.
     *
     * @category Authentication Service Methods
     */
    async createUser(user, options) {
        const url = appendToUrl(options?.url ?? this._url, "/users");
        const body = user;
        if (!requestValidation_js_1.AuthenticationServiceRequestValidation.validateCreateUserInput(body))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.AuthenticationServiceRequestValidation.validateCreateUserInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: "POST",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ],
            body: JSON.stringify(body)
        });
        if (!requestValidation_js_1.AuthenticationServiceRequestValidation.validateCreateUserOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.AuthenticationServiceRequestValidation.validateCreateUserOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    } /**
     * Get a user
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The JSON respresentation of the user.
     *
     * @category Authentication Service Methods
     */
    async getUser(url, options) {
        const urlSuffix = '/users/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [user_id,] = validateUrl(url, '/users/{}');
        const parameters = {
            user_id: user_id,
        };
        if (!requestValidation_js_1.AuthenticationServiceRequestValidation.validateGetUserInput(parameters))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.AuthenticationServiceRequestValidation.validateGetUserInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: "GET",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ]
        });
        if (!requestValidation_js_1.AuthenticationServiceRequestValidation.validateGetUserOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.AuthenticationServiceRequestValidation.validateGetUserOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    } /**
     * Update a user
     *
     * @param url
     * Url of the resource to be accessed.
     * @param user
     * Updated user
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The JSON representation of the updated user.
     *
     * @category Authentication Service Methods
     */
    async updateUser(url, user, options) {
        const urlSuffix = '/users/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [user_id,] = validateUrl(url, '/users/{}');
        const body = user;
        const parameters = {
            user_id: user_id,
        };
        if (!requestValidation_js_1.AuthenticationServiceRequestValidation.validateUpdateUserInput(parameters, body))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.AuthenticationServiceRequestValidation.validateUpdateUserInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: "PATCH",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ],
            body: JSON.stringify(body)
        });
        if (!requestValidation_js_1.AuthenticationServiceRequestValidation.validateUpdateUserOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.AuthenticationServiceRequestValidation.validateUpdateUserOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    } /**
     * Delete a user
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * User deleted successfully
     *
     * @category Authentication Service Methods
     */
    async deleteUser(url, options) {
        const urlSuffix = '/users/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [user_id,] = validateUrl(url, '/users/{}');
        const parameters = {
            user_id: user_id,
        };
        if (!requestValidation_js_1.AuthenticationServiceRequestValidation.validateDeleteUserInput(parameters))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.AuthenticationServiceRequestValidation.validateDeleteUserInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: "DELETE",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ]
        });
        if (!requestValidation_js_1.AuthenticationServiceRequestValidation.validateDeleteUserOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.AuthenticationServiceRequestValidation.validateDeleteUserOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
    } /**
     * Get identity
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The JSON representation of the corresponding user.
     *
     * @category Authentication Service Methods
     */
    async getIdentity(options) {
        const url = appendToUrl(this._url, "/identity");
        if (!requestValidation_js_1.AuthenticationServiceRequestValidation.validateGetIdentityInput())
            throw new ValidationError('Request validation failed!', requestValidation_js_1.AuthenticationServiceRequestValidation.validateGetIdentityInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: "GET",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ]
        });
        if (!requestValidation_js_1.AuthenticationServiceRequestValidation.validateGetIdentityOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.AuthenticationServiceRequestValidation.validateGetIdentityOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    } /**
     * Update identity
     *
     * @param user
     * Updated identity.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The JSON representation of the updated user.
     *
     * @category Authentication Service Methods
     */
    async updateIdentity(user, options) {
        const url = appendToUrl(this._url, "/identity");
        const body = user;
        if (!requestValidation_js_1.AuthenticationServiceRequestValidation.validateUpdateIdentityInput(body))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.AuthenticationServiceRequestValidation.validateUpdateIdentityInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: "PATCH",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ],
            body: JSON.stringify(body)
        });
        if (!requestValidation_js_1.AuthenticationServiceRequestValidation.validateUpdateIdentityOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.AuthenticationServiceRequestValidation.validateUpdateIdentityOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    } /**
     * Create a new token
     *
     * @param body
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The token was created successfully.
     *
     * @category Authentication Service Methods
     */
    async createToken(body, options) {
        const url = appendToUrl(this._url, "/token");
        if (!requestValidation_js_1.AuthenticationServiceRequestValidation.validateCreateTokenInput(body))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.AuthenticationServiceRequestValidation.validateCreateTokenInput.errors);
        const response = await this.fetch(url, {
            method: "POST",
            headers: [
                ["Content-Type", "application/json"],
                ...(options?.headers ?? [])
            ],
            body: JSON.stringify(body)
        });
        if (!requestValidation_js_1.AuthenticationServiceRequestValidation.validateCreateTokenOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.AuthenticationServiceRequestValidation.validateCreateTokenOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    } /**
     * List devices
     *
     * @param options.url
     * Url of the Device Service to be used.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The list of all devices.
     *
     * @category Device Service Methods
     */
    async listDevices(options) {
        const url = appendToUrl(options?.url ?? this._url, "/devices");
        if (!requestValidation_js_1.DeviceServiceRequestValidation.validateListDevicesInput())
            throw new ValidationError('Request validation failed!', requestValidation_js_1.DeviceServiceRequestValidation.validateListDevicesInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: "GET",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ]
        });
        if (!requestValidation_js_1.DeviceServiceRequestValidation.validateListDevicesOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.DeviceServiceRequestValidation.validateListDevicesOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    } /**
     * Create a new device
     *
     * @param device
     * The device to be created.
     * @param options.changedUrl
     * **An URL that will be called once the device changes.**
     *
     * Once the device was given a changedUrl parameter the contained URL will be called every time a device is changed
     * as long as every callback resolves with a successful status code.
     *
     * If the callback fails the url MIGHT not be called in the future.
     *
     * There can be multiple callbacks registered with the same device.
     * @param options.url
     * Url of the Device Service to be used.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The device was created. A JSON representation of the new device is returned.
     *
     * @category Device Service Methods
     */
    async createDevice(device, options) {
        const url = appendToUrl(options?.url ?? this._url, "/devices");
        const body = device;
        const parameters = {
            changedUrl: options?.changedUrl,
        };
        const query = [];
        if (parameters["changedUrl"])
            query.push(["changedUrl", parameters["changedUrl"].toString()]);
        if (!requestValidation_js_1.DeviceServiceRequestValidation.validateCreateDeviceInput(parameters, body))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.DeviceServiceRequestValidation.validateCreateDeviceInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url + '?' + new URLSearchParams(query), {
            method: "POST",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ],
            body: JSON.stringify(body)
        });
        if (!requestValidation_js_1.DeviceServiceRequestValidation.validateCreateDeviceOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.DeviceServiceRequestValidation.validateCreateDeviceOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    } /**
     * View a registered device
     *
     * @param url
     * Url of the resource to be accessed.
     * @param options.flat_group
     * If true the returned device group will only contain concrete devices. I.e. any subgroups will be flattend.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The JSON Representation of the device.
     *
     * @category Device Service Methods
     */
    async getDevice(url, options) {
        const urlSuffix = '/devices/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [device_id,] = validateUrl(url, '/devices/{}');
        const parameters = {
            device_id: device_id,
            flat_group: options?.flat_group,
        };
        const query = [];
        if (parameters["flat_group"])
            query.push(["flat_group", parameters["flat_group"].toString()]);
        if (!requestValidation_js_1.DeviceServiceRequestValidation.validateGetDeviceInput(parameters))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.DeviceServiceRequestValidation.validateGetDeviceInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url + '?' + new URLSearchParams(query), {
            method: "GET",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ]
        });
        if (!requestValidation_js_1.DeviceServiceRequestValidation.validateGetDeviceOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.DeviceServiceRequestValidation.validateGetDeviceOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    } /**
     * Update an existing device
     *
     * @param url
     * Url of the resource to be accessed.
     * @param deviceUpdate
     * Updated device.
     * @param options.changedUrl
     * **An URL that will be called once the device changes.**
     *
     * Once the device was given a changedUrl parameter the contained URL will be called every time a device is changed
     * as long as every callback resolves with a successful status code.
     *
     * If the callback fails the url MIGHT not be called in the future.
     *
     * There can be multiple callbacks registered with the same device.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The JSON Representation of the changed device.
     *
     * @category Device Service Methods
     */
    async updateDevice(url, deviceUpdate, options) {
        const urlSuffix = '/devices/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [device_id,] = validateUrl(url, '/devices/{}');
        const body = deviceUpdate;
        const parameters = {
            device_id: device_id,
            changedUrl: options?.changedUrl,
        };
        const query = [];
        if (parameters["changedUrl"])
            query.push(["changedUrl", parameters["changedUrl"].toString()]);
        if (!requestValidation_js_1.DeviceServiceRequestValidation.validateUpdateDeviceInput(parameters, body))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.DeviceServiceRequestValidation.validateUpdateDeviceInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url + '?' + new URLSearchParams(query), {
            method: "PATCH",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ],
            body: JSON.stringify(body)
        });
        if (!requestValidation_js_1.DeviceServiceRequestValidation.validateUpdateDeviceOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.DeviceServiceRequestValidation.validateUpdateDeviceOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    } /**
     * Delete a registered device
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The device was deleted.
     *
     * @category Device Service Methods
     */
    async deleteDevice(url, options) {
        const urlSuffix = '/devices/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [device_id,] = validateUrl(url, '/devices/{}');
        const parameters = {
            device_id: device_id,
        };
        if (!requestValidation_js_1.DeviceServiceRequestValidation.validateDeleteDeviceInput(parameters))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.DeviceServiceRequestValidation.validateDeleteDeviceInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: "DELETE",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ]
        });
        if (!requestValidation_js_1.DeviceServiceRequestValidation.validateDeleteDeviceOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.DeviceServiceRequestValidation.validateDeleteDeviceOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
    } /**
     * Instantiate a cloud instantiable device
     *
     * @param url
     * Url of the resource to be accessed.
     * @param options.changedUrl
     * **An URL that will be called once the device changes.**
     *
     * Once the device was given a changedUrl parameter the contained URL will be called every time a device is changed
     * as long as every callback resolves with a successful status code.
     *
     * If the callback fails the url MIGHT not be called in the future.
     *
     * There can be multiple callbacks registered with the same device.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The JSON Representation of the created instance.
     *
     * @category Device Service Methods
     */
    async instantiateDevice(url, options) {
        const urlSuffix = '/devices/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [device_id,] = validateUrl(url, '/devices/{}');
        const parameters = {
            device_id: device_id,
            changedUrl: options?.changedUrl,
        };
        const query = [];
        if (parameters["changedUrl"])
            query.push(["changedUrl", parameters["changedUrl"].toString()]);
        if (!requestValidation_js_1.DeviceServiceRequestValidation.validateInstantiateDeviceInput(parameters))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.DeviceServiceRequestValidation.validateInstantiateDeviceInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url + '?' + new URLSearchParams(query), {
            method: "POST",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ]
        });
        if (!requestValidation_js_1.DeviceServiceRequestValidation.validateInstantiateDeviceOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.DeviceServiceRequestValidation.validateInstantiateDeviceOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    } /**
     * Get the availability of a device
     *
     * @param url
     * Url of the resource to be accessed.
     * @param options.startTime
     * Start time of the requested availability
     * @param options.endTime
     * End time of the requested availability
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The JSON Representation of the availability of the device.
     *
     * @category Device Service Methods
     */
    async getDeviceAvailability(url, options) {
        const urlSuffix = '/devices/{}/availability'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [device_id,] = validateUrl(url, '/devices/{}/availability');
        const parameters = {
            device_id: device_id,
            startTime: options?.startTime,
            endTime: options?.endTime,
        };
        const query = [];
        if (parameters["startTime"])
            query.push(["startTime", parameters["startTime"].toString()]);
        if (parameters["endTime"])
            query.push(["endTime", parameters["endTime"].toString()]);
        if (!requestValidation_js_1.DeviceServiceRequestValidation.validateGetDeviceAvailabilityInput(parameters))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.DeviceServiceRequestValidation.validateGetDeviceAvailabilityInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url + '?' + new URLSearchParams(query), {
            method: "GET",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ]
        });
        if (!requestValidation_js_1.DeviceServiceRequestValidation.validateGetDeviceAvailabilityOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.DeviceServiceRequestValidation.validateGetDeviceAvailabilityOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    } /**
     * Delete the availability rules of a device
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The availability rules of the device were deleted successfully.
     *
     * @category Device Service Methods
     */
    async deleteDeviceAvailabilityRules(url, options) {
        const urlSuffix = '/devices/{}/availability'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [device_id,] = validateUrl(url, '/devices/{}/availability');
        const parameters = {
            device_id: device_id,
        };
        if (!requestValidation_js_1.DeviceServiceRequestValidation.validateDeleteDeviceAvailabilityRulesInput(parameters))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.DeviceServiceRequestValidation.validateDeleteDeviceAvailabilityRulesInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: "DELETE",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ]
        });
        if (!requestValidation_js_1.DeviceServiceRequestValidation.validateDeleteDeviceAvailabilityRulesOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.DeviceServiceRequestValidation.validateDeleteDeviceAvailabilityRulesOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
    } /**
     * Add availability rules for a device
     *
     * @param url
     * Url of the resource to be accessed.
     * @param availabilityRules
     * The availability rules to be applied.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The JSON Representation of the changed availability.
     *
     * @category Device Service Methods
     */
    async addDeviceAvailabilityRules(url, availabilityRules, options) {
        const urlSuffix = '/devices/{}/availability'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [device_id,] = validateUrl(url, '/devices/{}/availability');
        const body = availabilityRules;
        const parameters = {
            device_id: device_id,
        };
        if (!requestValidation_js_1.DeviceServiceRequestValidation.validateAddDeviceAvailabilityRulesInput(parameters, body))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.DeviceServiceRequestValidation.validateAddDeviceAvailabilityRulesInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: "POST",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ],
            body: JSON.stringify(body)
        });
        if (!requestValidation_js_1.DeviceServiceRequestValidation.validateAddDeviceAvailabilityRulesOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.DeviceServiceRequestValidation.validateAddDeviceAvailabilityRulesOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    } /**
     * Create new websocket token for device
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The newly generated websocket token for the device.
     *
     * @category Device Service Methods
     */
    async createWebsocketToken(url, options) {
        const urlSuffix = '/devices/{}/websocket'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [device_id,] = validateUrl(url, '/devices/{}/websocket');
        const parameters = {
            device_id: device_id,
        };
        if (!requestValidation_js_1.DeviceServiceRequestValidation.validateCreateWebsocketTokenInput(parameters))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.DeviceServiceRequestValidation.validateCreateWebsocketTokenInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: "POST",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ]
        });
        if (!requestValidation_js_1.DeviceServiceRequestValidation.validateCreateWebsocketTokenOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.DeviceServiceRequestValidation.validateCreateWebsocketTokenOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    } /**
     * Send signaling message to device
     *
     * @param url
     * Url of the resource to be accessed.
     * @param sigMessage
     * The signaling message to be sent.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The signaling message was delivered successfully.
     *
     * @category Device Service Methods
     */
    async sendSignalingMessage(url, sigMessage, options) {
        const urlSuffix = '/devices/{}/signaling'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [device_id,] = validateUrl(url, '/devices/{}/signaling');
        const body = sigMessage;
        const parameters = {
            device_id: device_id,
        };
        if (!requestValidation_js_1.DeviceServiceRequestValidation.validateSendSignalingMessageInput(parameters, body))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.DeviceServiceRequestValidation.validateSendSignalingMessageInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: "POST",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ],
            body: JSON.stringify(body)
        });
        if (!requestValidation_js_1.DeviceServiceRequestValidation.validateSendSignalingMessageOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.DeviceServiceRequestValidation.validateSendSignalingMessageOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
    } /**
     * List Peer Connection
     *
     * @param options.url
     * Url of the Device Service to be used.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The list of all peerconnections.
     *
     * @category Device Service Methods
     */
    async listPeerconnections(options) {
        const url = appendToUrl(options?.url ?? this._url, "/peerconnections");
        if (!requestValidation_js_1.DeviceServiceRequestValidation.validateListPeerconnectionsInput())
            throw new ValidationError('Request validation failed!', requestValidation_js_1.DeviceServiceRequestValidation.validateListPeerconnectionsInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: "GET",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ]
        });
        if (!requestValidation_js_1.DeviceServiceRequestValidation.validateListPeerconnectionsOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.DeviceServiceRequestValidation.validateListPeerconnectionsOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    } /**
     * Create a new Peer Connection
     *
     * @param peerconnection
     * The peerconnection to be created.
     * @param options.closedUrl
     * An URL that will be called once the peer connection is closed.
     * @param options.statusChangedUrl
     * An URL that will be called if the status of the peerconnection changes.
     * @param options.url
     * Url of the Device Service to be used.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * 201: The peerconnection was created. A JSON representation of the new peer connection is returned.
     * 202: The device service will attempt to create the peerconnection as soon as possible.
     *
     * @category Device Service Methods
     */
    async createPeerconnection(peerconnection, options) {
        const url = appendToUrl(options?.url ?? this._url, "/peerconnections");
        const body = peerconnection;
        const parameters = {
            closedUrl: options?.closedUrl,
            statusChangedUrl: options?.statusChangedUrl,
        };
        const query = [];
        if (parameters["closedUrl"])
            query.push(["closedUrl", parameters["closedUrl"].toString()]);
        if (parameters["statusChangedUrl"])
            query.push(["statusChangedUrl", parameters["statusChangedUrl"].toString()]);
        if (!requestValidation_js_1.DeviceServiceRequestValidation.validateCreatePeerconnectionInput(parameters, body))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.DeviceServiceRequestValidation.validateCreatePeerconnectionInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url + '?' + new URLSearchParams(query), {
            method: "POST",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ],
            body: JSON.stringify(body)
        });
        if (!requestValidation_js_1.DeviceServiceRequestValidation.validateCreatePeerconnectionOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.DeviceServiceRequestValidation.validateCreatePeerconnectionOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    } /**
     * View a peer connection
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The JSON Representation of the device peer connection.
     *
     * @category Device Service Methods
     */
    async getPeerconnection(url, options) {
        const urlSuffix = '/peerconnections/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [peerconnection_id,] = validateUrl(url, '/peerconnections/{}');
        const parameters = {
            peerconnection_id: peerconnection_id,
        };
        if (!requestValidation_js_1.DeviceServiceRequestValidation.validateGetPeerconnectionInput(parameters))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.DeviceServiceRequestValidation.validateGetPeerconnectionInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: "GET",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ]
        });
        if (!requestValidation_js_1.DeviceServiceRequestValidation.validateGetPeerconnectionOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.DeviceServiceRequestValidation.validateGetPeerconnectionOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    } /**
     * Delete a peer connection
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * 202: The peerconnection will be deleted as soon as it's closed.
     * 204: The peerconnection was deleted successfully.
     *
     * @category Device Service Methods
     */
    async deletePeerconnection(url, options) {
        const urlSuffix = '/peerconnections/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [peerconnection_id,] = validateUrl(url, '/peerconnections/{}');
        const parameters = {
            peerconnection_id: peerconnection_id,
        };
        if (!requestValidation_js_1.DeviceServiceRequestValidation.validateDeletePeerconnectionInput(parameters))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.DeviceServiceRequestValidation.validateDeletePeerconnectionInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: "DELETE",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ]
        });
        if (!requestValidation_js_1.DeviceServiceRequestValidation.validateDeletePeerconnectionOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.DeviceServiceRequestValidation.validateDeletePeerconnectionOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
    } /**
     * Sets the peerconnection status of a single device.
     *
     * @param url
     * Url of the resource to be accessed.
     * @param body
     * The JSON Representation of the device peer connection.
     * @param device_url
     * URL of the device
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * OK.
     *
     * @category Device Service Methods
     */
    async patchPeerconnectionDeviceStatus(url, body, device_url, options) {
        const urlSuffix = '/peerconnections/{}/device_status'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [peerconnection_id,] = validateUrl(url, '/peerconnections/{}/device_status');
        const parameters = {
            peerconnection_id: peerconnection_id,
            device_url: device_url,
        };
        const query = [];
        if (parameters["device_url"])
            query.push(["device_url", parameters["device_url"].toString()]);
        if (!requestValidation_js_1.DeviceServiceRequestValidation.validatePatchPeerconnectionDeviceStatusInput(parameters, body))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.DeviceServiceRequestValidation.validatePatchPeerconnectionDeviceStatusInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url + '?' + new URLSearchParams(query), {
            method: "PATCH",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ],
            body: JSON.stringify(body)
        });
        if (!requestValidation_js_1.DeviceServiceRequestValidation.validatePatchPeerconnectionDeviceStatusOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.DeviceServiceRequestValidation.validatePatchPeerconnectionDeviceStatusOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
    } /**
     * List experiments
     *
     * @param options.url
     * Url of the Experiment Service to be used.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The list of all experiments.
     *
     * @category Experiment Service Methods
     */
    async listExperiments(options) {
        const url = appendToUrl(options?.url ?? this._url, "/experiments");
        if (!requestValidation_js_1.ExperimentServiceRequestValidation.validateListExperimentsInput())
            throw new ValidationError('Request validation failed!', requestValidation_js_1.ExperimentServiceRequestValidation.validateListExperimentsInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: "GET",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ]
        });
        if (!requestValidation_js_1.ExperimentServiceRequestValidation.validateListExperimentsOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.ExperimentServiceRequestValidation.validateListExperimentsOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    } /**
     * Create a new experiment
     *
     * @param experiment
     * The experiment to be created.
     * @param options.changedURL
     * An URL that will be called when the experiment status changes.
     * @param options.url
     * Url of the Experiment Service to be used.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * 201: The experiment was created. A JSON representation of the new experiment is returned.
     * 202: The experiment was created and will be set to running eventually. A JSON representation of the new experiment is returned.
     *
     * @category Experiment Service Methods
     */
    async createExperiment(experiment, options) {
        const url = appendToUrl(options?.url ?? this._url, "/experiments");
        const body = experiment;
        const parameters = {
            changedURL: options?.changedURL,
        };
        const query = [];
        if (parameters["changedURL"])
            query.push(["changedURL", parameters["changedURL"].toString()]);
        if (!requestValidation_js_1.ExperimentServiceRequestValidation.validateCreateExperimentInput(parameters, body))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.ExperimentServiceRequestValidation.validateCreateExperimentInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url + '?' + new URLSearchParams(query), {
            method: "POST",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ],
            body: JSON.stringify(body)
        });
        if (!requestValidation_js_1.ExperimentServiceRequestValidation.validateCreateExperimentOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.ExperimentServiceRequestValidation.validateCreateExperimentOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    } /**
     * View an experiment.
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The JSON Representation of the experiment
     *
     * @category Experiment Service Methods
     */
    async getExperiment(url, options) {
        const urlSuffix = '/experiments/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [experiment_id,] = validateUrl(url, '/experiments/{}');
        const parameters = {
            experiment_id: experiment_id,
        };
        if (!requestValidation_js_1.ExperimentServiceRequestValidation.validateGetExperimentInput(parameters))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.ExperimentServiceRequestValidation.validateGetExperimentInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: "GET",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ]
        });
        if (!requestValidation_js_1.ExperimentServiceRequestValidation.validateGetExperimentOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.ExperimentServiceRequestValidation.validateGetExperimentOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    } /**
     * Update an existing experiment.
     *
     * @param url
     * Url of the resource to be accessed.
     * @param experimentUpdate
     * Update the experiment
     * @param options.changedURL
     * An URL that will be called when the experiment status changes.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * 200: The JSON Representation of the changed experiment
     * 202: The JSON Representation of the changed experiment, that will be set to running eventually
     *
     * @category Experiment Service Methods
     */
    async updateExperiment(url, experimentUpdate, options) {
        const urlSuffix = '/experiments/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [experiment_id,] = validateUrl(url, '/experiments/{}');
        const body = experimentUpdate;
        const parameters = {
            experiment_id: experiment_id,
            changedURL: options?.changedURL,
        };
        const query = [];
        if (parameters["changedURL"])
            query.push(["changedURL", parameters["changedURL"].toString()]);
        if (!requestValidation_js_1.ExperimentServiceRequestValidation.validateUpdateExperimentInput(parameters, body))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.ExperimentServiceRequestValidation.validateUpdateExperimentInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url + '?' + new URLSearchParams(query), {
            method: "PATCH",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ],
            body: JSON.stringify(body)
        });
        if (!requestValidation_js_1.ExperimentServiceRequestValidation.validateUpdateExperimentOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.ExperimentServiceRequestValidation.validateUpdateExperimentOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    } /**
     * Delete an experiment
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The experiment was deleted.
     *
     * @category Experiment Service Methods
     */
    async deleteExperiment(url, options) {
        const urlSuffix = '/experiments/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [experiment_id,] = validateUrl(url, '/experiments/{}');
        const parameters = {
            experiment_id: experiment_id,
        };
        if (!requestValidation_js_1.ExperimentServiceRequestValidation.validateDeleteExperimentInput(parameters))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.ExperimentServiceRequestValidation.validateDeleteExperimentInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: "DELETE",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ]
        });
        if (!requestValidation_js_1.ExperimentServiceRequestValidation.validateDeleteExperimentOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.ExperimentServiceRequestValidation.validateDeleteExperimentOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
    } /**
     * List templates
     *
     * @param options.url
     * Url of the Experiment Service to be used.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The list of all templates.
     *
     * @category Experiment Service Methods
     */
    async listTemplate(options) {
        const url = appendToUrl(options?.url ?? this._url, "/templates");
        if (!requestValidation_js_1.ExperimentServiceRequestValidation.validateListTemplateInput())
            throw new ValidationError('Request validation failed!', requestValidation_js_1.ExperimentServiceRequestValidation.validateListTemplateInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: "GET",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ]
        });
        if (!requestValidation_js_1.ExperimentServiceRequestValidation.validateListTemplateOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.ExperimentServiceRequestValidation.validateListTemplateOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    } /**
     * Create a new template
     *
     * @param template
     * The template to be created.
     * @param options.url
     * Url of the Experiment Service to be used.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * 201: The template was created. A JSON representation of the new template is returned.
     * 202: The template was created and will be set to running eventually. A JSON representation of the new template is returned.
     *
     * @category Experiment Service Methods
     */
    async createTemplate(template, options) {
        const url = appendToUrl(options?.url ?? this._url, "/templates");
        const body = template;
        if (!requestValidation_js_1.ExperimentServiceRequestValidation.validateCreateTemplateInput(body))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.ExperimentServiceRequestValidation.validateCreateTemplateInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: "POST",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ],
            body: JSON.stringify(body)
        });
        if (!requestValidation_js_1.ExperimentServiceRequestValidation.validateCreateTemplateOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.ExperimentServiceRequestValidation.validateCreateTemplateOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    } /**
     * View an template.
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The JSON Representation of the template
     *
     * @category Experiment Service Methods
     */
    async getTemplate(url, options) {
        const urlSuffix = '/templates/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [template_id,] = validateUrl(url, '/templates/{}');
        const parameters = {
            template_id: template_id,
        };
        if (!requestValidation_js_1.ExperimentServiceRequestValidation.validateGetTemplateInput(parameters))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.ExperimentServiceRequestValidation.validateGetTemplateInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: "GET",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ]
        });
        if (!requestValidation_js_1.ExperimentServiceRequestValidation.validateGetTemplateOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.ExperimentServiceRequestValidation.validateGetTemplateOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    } /**
     * Update an existing template.
     *
     * @param url
     * Url of the resource to be accessed.
     * @param templateUpdate
     * Update the template
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * 200: The JSON Representation of the changed template
     * 202: The JSON Representation of the changed template, that will be set to running eventually
     *
     * @category Experiment Service Methods
     */
    async updateTemplate(url, templateUpdate, options) {
        const urlSuffix = '/templates/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [template_id,] = validateUrl(url, '/templates/{}');
        const body = templateUpdate;
        const parameters = {
            template_id: template_id,
        };
        if (!requestValidation_js_1.ExperimentServiceRequestValidation.validateUpdateTemplateInput(parameters, body))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.ExperimentServiceRequestValidation.validateUpdateTemplateInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: "PATCH",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ],
            body: JSON.stringify(body)
        });
        if (!requestValidation_js_1.ExperimentServiceRequestValidation.validateUpdateTemplateOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.ExperimentServiceRequestValidation.validateUpdateTemplateOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    } /**
     * Delete an template
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The template was deleted.
     *
     * @category Experiment Service Methods
     */
    async deleteTemplate(url, options) {
        const urlSuffix = '/templates/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [template_id,] = validateUrl(url, '/templates/{}');
        const parameters = {
            template_id: template_id,
        };
        if (!requestValidation_js_1.ExperimentServiceRequestValidation.validateDeleteTemplateInput(parameters))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.ExperimentServiceRequestValidation.validateDeleteTemplateInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: "DELETE",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ]
        });
        if (!requestValidation_js_1.ExperimentServiceRequestValidation.validateDeleteTemplateOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.ExperimentServiceRequestValidation.validateDeleteTemplateOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
    } /**
     * List institutions
     *
     * @param options.url
     * Url of the Federation Service to be used.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The list of all institutions.
     *
     * @category Federation Service Methods
     */
    async listInstitutions(options) {
        const url = appendToUrl(options?.url ?? this._url, "/institutions");
        if (!requestValidation_js_1.FederationServiceRequestValidation.validateListInstitutionsInput())
            throw new ValidationError('Request validation failed!', requestValidation_js_1.FederationServiceRequestValidation.validateListInstitutionsInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: "GET",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ]
        });
        if (!requestValidation_js_1.FederationServiceRequestValidation.validateListInstitutionsOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.FederationServiceRequestValidation.validateListInstitutionsOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    } /**
     * Create a new institution
     *
     * @param institution
     * The institution to be created.
     * @param options.url
     * Url of the Federation Service to be used.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The institution was created. A JSON representation of the new institution is returned.
     *
     * @category Federation Service Methods
     */
    async createInstitution(institution, options) {
        const url = appendToUrl(options?.url ?? this._url, "/institutions");
        const body = institution;
        if (!requestValidation_js_1.FederationServiceRequestValidation.validateCreateInstitutionInput(body))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.FederationServiceRequestValidation.validateCreateInstitutionInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: "POST",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ],
            body: JSON.stringify(body)
        });
        if (!requestValidation_js_1.FederationServiceRequestValidation.validateCreateInstitutionOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.FederationServiceRequestValidation.validateCreateInstitutionOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    } /**
     * View an institution.
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The JSON Representation of the institution
     *
     * @category Federation Service Methods
     */
    async getInstitution(url, options) {
        const urlSuffix = '/institutions/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [institution_id,] = validateUrl(url, '/institutions/{}');
        const parameters = {
            institution_id: institution_id,
        };
        if (!requestValidation_js_1.FederationServiceRequestValidation.validateGetInstitutionInput(parameters))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.FederationServiceRequestValidation.validateGetInstitutionInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: "GET",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ]
        });
        if (!requestValidation_js_1.FederationServiceRequestValidation.validateGetInstitutionOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.FederationServiceRequestValidation.validateGetInstitutionOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    } /**
     * Update an institution.
     *
     * @param url
     * Url of the resource to be accessed.
     * @param institution
     * Update the institution
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The JSON Representation of the changed institution
     *
     * @category Federation Service Methods
     */
    async updateInstitution(url, institution, options) {
        const urlSuffix = '/institutions/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [institution_id,] = validateUrl(url, '/institutions/{}');
        const body = institution;
        const parameters = {
            institution_id: institution_id,
        };
        if (!requestValidation_js_1.FederationServiceRequestValidation.validateUpdateInstitutionInput(parameters, body))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.FederationServiceRequestValidation.validateUpdateInstitutionInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: "PATCH",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ],
            body: JSON.stringify(body)
        });
        if (!requestValidation_js_1.FederationServiceRequestValidation.validateUpdateInstitutionOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.FederationServiceRequestValidation.validateUpdateInstitutionOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    } /**
     * Delete an institution
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * The institution was delete.
     *
     * @category Federation Service Methods
     */
    async deleteInstitution(url, options) {
        const urlSuffix = '/institutions/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [institution_id,] = validateUrl(url, '/institutions/{}');
        const parameters = {
            institution_id: institution_id,
        };
        if (!requestValidation_js_1.FederationServiceRequestValidation.validateDeleteInstitutionInput(parameters))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.FederationServiceRequestValidation.validateDeleteInstitutionInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: "DELETE",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ]
        });
        if (!requestValidation_js_1.FederationServiceRequestValidation.validateDeleteInstitutionOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.FederationServiceRequestValidation.validateDeleteInstitutionOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
    } /**
     *
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * Returns the registration parameters for the LTI service.
     *
     * @category LTI Service Methods
     */
    async listPlatform(options) {
        const url = appendToUrl(this._url, "/lti/platform");
        if (!requestValidation_js_1.LTIServiceRequestValidation.validateListPlatformInput())
            throw new ValidationError('Request validation failed!', requestValidation_js_1.LTIServiceRequestValidation.validateListPlatformInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: "GET",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ]
        });
        if (!requestValidation_js_1.LTIServiceRequestValidation.validateListPlatformOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.LTIServiceRequestValidation.validateListPlatformOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    } /**
     *
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * Platform created
     *
     * @category LTI Service Methods
     */
    async registerPlatform(options) {
        const url = appendToUrl(this._url, "/lti/platform");
        if (!requestValidation_js_1.LTIServiceRequestValidation.validateRegisterPlatformInput())
            throw new ValidationError('Request validation failed!', requestValidation_js_1.LTIServiceRequestValidation.validateRegisterPlatformInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: "POST",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ]
        });
        if (!requestValidation_js_1.LTIServiceRequestValidation.validateRegisterPlatformOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.LTIServiceRequestValidation.validateRegisterPlatformOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    } /**
     *
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * Returns the platform.
     *
     * @category LTI Service Methods
     */
    async getPlatform(url, options) {
        const urlSuffix = '/lti/platform/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [platform_id,] = validateUrl(url, '/lti/platform/{}');
        const parameters = {
            platform_id: platform_id,
        };
        if (!requestValidation_js_1.LTIServiceRequestValidation.validateGetPlatformInput(parameters))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.LTIServiceRequestValidation.validateGetPlatformInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: "GET",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ]
        });
        if (!requestValidation_js_1.LTIServiceRequestValidation.validateGetPlatformOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.LTIServiceRequestValidation.validateGetPlatformOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    } /**
     *
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * Platform updated
     *
     * @category LTI Service Methods
     */
    async updatePlatform(url, options) {
        const urlSuffix = '/lti/platform/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [platform_id,] = validateUrl(url, '/lti/platform/{}');
        const parameters = {
            platform_id: platform_id,
        };
        if (!requestValidation_js_1.LTIServiceRequestValidation.validateUpdatePlatformInput(parameters))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.LTIServiceRequestValidation.validateUpdatePlatformInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: "PATCH",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ]
        });
        if (!requestValidation_js_1.LTIServiceRequestValidation.validateUpdatePlatformOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.LTIServiceRequestValidation.validateUpdatePlatformOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    } /**
     *
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * Platform deleted
     *
     * @category LTI Service Methods
     */
    async deletePlatform(url, options) {
        const urlSuffix = '/lti/platform/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [platform_id,] = validateUrl(url, '/lti/platform/{}');
        const parameters = {
            platform_id: platform_id,
        };
        if (!requestValidation_js_1.LTIServiceRequestValidation.validateDeletePlatformInput(parameters))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.LTIServiceRequestValidation.validateDeletePlatformInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: "DELETE",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ]
        });
        if (!requestValidation_js_1.LTIServiceRequestValidation.validateDeletePlatformOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.LTIServiceRequestValidation.validateDeletePlatformOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
    } /**
     *
     *
     * @param url
     * Url of the resource to be accessed.
     * @param body
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * returns the verified and parsed LTI message.
     *
     * @category LTI Service Methods
     */
    async ltiLogin(url, body, options) {
        const urlSuffix = '/lti/platform/{}/login'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [platform_id,] = validateUrl(url, '/lti/platform/{}/login');
        const parameters = {
            platform_id: platform_id,
        };
        if (!requestValidation_js_1.LTIServiceRequestValidation.validateLtiLoginInput(parameters, body))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.LTIServiceRequestValidation.validateLtiLoginInput.errors);
        const response = await this.fetch(url, {
            method: "POST",
            headers: [
                ["Content-Type", "application/json"],
                ...(options?.headers ?? [])
            ],
            body: JSON.stringify(body)
        });
        if (!requestValidation_js_1.LTIServiceRequestValidation.validateLtiLoginOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.LTIServiceRequestValidation.validateLtiLoginOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    } /**
     *
     *
     * @param url
     * Url of the resource to be accessed.
     * @param body
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * returns the verified and parsed LTI message.
     *
     * @category LTI Service Methods
     */
    async ltiLaunch(url, body, options) {
        const urlSuffix = '/lti/platform/{}/launch'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [platform_id,] = validateUrl(url, '/lti/platform/{}/launch');
        const parameters = {
            platform_id: platform_id,
        };
        if (!requestValidation_js_1.LTIServiceRequestValidation.validateLtiLaunchInput(parameters, body))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.LTIServiceRequestValidation.validateLtiLaunchInput.errors);
        const response = await this.fetch(url, {
            method: "POST",
            headers: [
                ["Content-Type", "application/json"],
                ...(options?.headers ?? [])
            ],
            body: JSON.stringify(body)
        });
        if (!requestValidation_js_1.LTIServiceRequestValidation.validateLtiLaunchOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.LTIServiceRequestValidation.validateLtiLaunchOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    } /**
     *
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * returns the JSON Web Key Set (JWKS) for the tool.
     *
     * @category LTI Service Methods
     */
    async ltiJwks(url, options) {
        const urlSuffix = '/lti/platform/{}/jwks'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [platform_id,] = validateUrl(url, '/lti/platform/{}/jwks');
        const parameters = {
            platform_id: platform_id,
        };
        if (!requestValidation_js_1.LTIServiceRequestValidation.validateLtiJwksInput(parameters))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.LTIServiceRequestValidation.validateLtiJwksInput.errors);
        const response = await this.fetch(url, {
            method: "GET",
            headers: [
                ["Content-Type", "application/json"],
                ...(options?.headers ?? [])
            ]
        });
        if (!requestValidation_js_1.LTIServiceRequestValidation.validateLtiJwksOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.LTIServiceRequestValidation.validateLtiJwksOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    } /**
     *
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * List all LTI-Resources.
     *
     * @category LTI Service Methods
     */
    async listResource(options) {
        const url = appendToUrl(this._url, "/lti/resource");
        if (!requestValidation_js_1.LTIServiceRequestValidation.validateListResourceInput())
            throw new ValidationError('Request validation failed!', requestValidation_js_1.LTIServiceRequestValidation.validateListResourceInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: "GET",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ]
        });
        if (!requestValidation_js_1.LTIServiceRequestValidation.validateListResourceOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.LTIServiceRequestValidation.validateListResourceOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    } /**
     *
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * Returns the resource.
     *
     * @category LTI Service Methods
     */
    async getResource(url, options) {
        const urlSuffix = '/lti/resource/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [resource_id,] = validateUrl(url, '/lti/resource/{}');
        const parameters = {
            resource_id: resource_id,
        };
        if (!requestValidation_js_1.LTIServiceRequestValidation.validateGetResourceInput(parameters))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.LTIServiceRequestValidation.validateGetResourceInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: "GET",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ]
        });
        if (!requestValidation_js_1.LTIServiceRequestValidation.validateGetResourceOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.LTIServiceRequestValidation.validateGetResourceOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    } /**
     *
     *
     * @param url
     * Url of the resource to be accessed.
     * @param body
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * resource updated
     *
     * @category LTI Service Methods
     */
    async updateResource(url, body, options) {
        const urlSuffix = '/lti/resource/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [resource_id,] = validateUrl(url, '/lti/resource/{}');
        const parameters = {
            resource_id: resource_id,
        };
        if (!requestValidation_js_1.LTIServiceRequestValidation.validateUpdateResourceInput(parameters, body))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.LTIServiceRequestValidation.validateUpdateResourceInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: "PATCH",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ],
            body: JSON.stringify(body)
        });
        if (!requestValidation_js_1.LTIServiceRequestValidation.validateUpdateResourceOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.LTIServiceRequestValidation.validateUpdateResourceOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    } /**
     *
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * resource deleted
     *
     * @category LTI Service Methods
     */
    async deleteResource(url, options) {
        const urlSuffix = '/lti/resource/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [resource_id,] = validateUrl(url, '/lti/resource/{}');
        const parameters = {
            resource_id: resource_id,
        };
        if (!requestValidation_js_1.LTIServiceRequestValidation.validateDeleteResourceInput(parameters))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.LTIServiceRequestValidation.validateDeleteResourceInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: "DELETE",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ]
        });
        if (!requestValidation_js_1.LTIServiceRequestValidation.validateDeleteResourceOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.LTIServiceRequestValidation.validateDeleteResourceOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
    } /**
     *
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * List all students of the resource.
     *
     * @category LTI Service Methods
     */
    async listResourceStudents(url, options) {
        const urlSuffix = '/lti/resource/{}/students'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [resource_id,] = validateUrl(url, '/lti/resource/{}/students');
        const parameters = {
            resource_id: resource_id,
        };
        if (!requestValidation_js_1.LTIServiceRequestValidation.validateListResourceStudentsInput(parameters))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.LTIServiceRequestValidation.validateListResourceStudentsInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: "GET",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ]
        });
        if (!requestValidation_js_1.LTIServiceRequestValidation.validateListResourceStudentsOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.LTIServiceRequestValidation.validateListResourceStudentsOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    } /**
     *
     *
     * @param url
     * Url of the resource to be accessed.
     * @param body
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * Returns the students.
     *
     * @category LTI Service Methods
     */
    async updateResourceStudents(url, body, options) {
        const urlSuffix = '/lti/resource/{}/students'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [resource_id,] = validateUrl(url, '/lti/resource/{}/students');
        const parameters = {
            resource_id: resource_id,
        };
        if (!requestValidation_js_1.LTIServiceRequestValidation.validateUpdateResourceStudentsInput(parameters, body))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.LTIServiceRequestValidation.validateUpdateResourceStudentsInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: "PATCH",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ],
            body: JSON.stringify(body)
        });
        if (!requestValidation_js_1.LTIServiceRequestValidation.validateUpdateResourceStudentsOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.LTIServiceRequestValidation.validateUpdateResourceStudentsOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    } /**
     *
     *
     * @param url
     * Url of the resource to be accessed.
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * Returns the student.
     *
     * @category LTI Service Methods
     */
    async getResourceStudent(url, options) {
        const urlSuffix = '/lti/resource/{}/students/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [resource_id, student_id,] = validateUrl(url, '/lti/resource/{}/students/{}');
        const parameters = {
            resource_id: resource_id,
            student_id: student_id,
        };
        if (!requestValidation_js_1.LTIServiceRequestValidation.validateGetResourceStudentInput(parameters))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.LTIServiceRequestValidation.validateGetResourceStudentInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: "GET",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ]
        });
        if (!requestValidation_js_1.LTIServiceRequestValidation.validateGetResourceStudentOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.LTIServiceRequestValidation.validateGetResourceStudentOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    } /**
     *
     *
     * @param url
     * Url of the resource to be accessed.
     * @param student
     *
     * @throws {@link FetchError | FetchError }
     * Thrown if fetch fails.
     * @throws {@link ValidationError | ValidationError }
     * Thrown if the request/response validation fails.
     * @throws {@link InvalidUrlError | InvalidUrlError }
     * Thrown if the provided url is not valid for this request.
     * @throws {@link UnsuccessfulRequestError | UnsuccessfulRequestError }
     * Thrown if response is validated but has status greater than or equal to 400.
     *
     * @returns
     * Returns the student.
     *
     * @category LTI Service Methods
     */
    async updateResourceStudent(url, student, options) {
        const urlSuffix = '/lti/resource/{}/students/{}'.split('{}').at(-1) ?? '';
        if (!url.endsWith(urlSuffix))
            url = appendToUrl(url, urlSuffix);
        const [resource_id, student_id,] = validateUrl(url, '/lti/resource/{}/students/{}');
        const body = student;
        const parameters = {
            resource_id: resource_id,
            student_id: student_id,
        };
        if (!requestValidation_js_1.LTIServiceRequestValidation.validateUpdateResourceStudentInput(parameters, body))
            throw new ValidationError('Request validation failed!', requestValidation_js_1.LTIServiceRequestValidation.validateUpdateResourceStudentInput.errors);
        const authorization = `Bearer ${this._accessToken}`;
        const response = await this.fetch(url, {
            method: "PATCH",
            headers: [
                ["Content-Type", "application/json"],
                ["Authorization", authorization],
                ...(options?.headers ?? [])
            ],
            body: JSON.stringify(body)
        });
        if (!requestValidation_js_1.LTIServiceRequestValidation.validateUpdateResourceStudentOutput(response))
            throw new ValidationError('Response validation failed!', requestValidation_js_1.LTIServiceRequestValidation.validateUpdateResourceStudentOutput.errors);
        if ((0, types_js_1.isErrorResponse)(response))
            throw new UnsuccessfulRequestError(`Server returned response with status ${response.status}`, response);
        return response.body;
    }
}
exports.APIClient = APIClient;
//# sourceMappingURL=client.js.map